##############################################################################
#
# Copyright (c) 2002 Nik Kim <fafhrd@datacom.kz> All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import Globals, traceback
from Globals import InitializeClass
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCorePermissions import \
    View, ModifyPortalContent, ListFolderContents, AccessContentsInformation
from Products.CMFCore.utils import _getViewFor
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.Formulator.Errors import ValidationError, FormValidationError
from Products.Formulator.Form import ZMIForm, Form
from Products.Formulator.XMLToForm import XMLToForm
from CMFField import CMFField
from Products.CMFPlone.PloneFolder import PloneFolder

factory_type_information = (
    {   'id'             : 'Form',
        'meta_type'      : 'CMF Form',
        'description'    : """Form""",
        'icon'           : 'Form.gif',
        'product'        : 'CMFFormulator',
        'factory'        : 'manage_addFormulatorForm',
        'immediate_view' : 'form_edit_form',
        'filter_content_types' : 1,
        'actions'        :
        ( { 'id'         : 'view',
            'name'       : 'View',
            'action'     : 'form_view',
            'permissions': (View,),
            'category'   : 'object',
            },
    	  { 'id'         : 'folderlisting',
            'name'       : 'Folder Listing',
            'action'     : 'folder_listing',
            'permissions': (AccessContentsInformation,),
            'category'   : 'folder',
            },
          { 'id'         : 'edit',
            'name'       : 'Edit',
            'action'     : 'form_edit_form',
            'permissions': (ModifyPortalContent,),
            'category'   : 'object',
            },
          { 'id'         : 'groups',
            'name'       : 'Groups',
            'action'     : 'form_groups_form',
            'permissions': (ModifyPortalContent,),
            'category'   : 'object',
            },
          { 'id'         : 'metadata',
            'name'       : 'Metadata',
            'action'     : 'metadata_edit_form',
            'permissions': (ModifyPortalContent, ),
            'category'   : 'object',
            },
          )
        },
    )

try:
    # i don't now what to do with this, plone use templates with this names
    
    del Form.header
    del Form.footer
except:
    pass

class CMFForm(PortalFolder, PortalContent, DefaultDublinCoreImpl, ZMIForm):
    ''' '''
    meta_type = 'CMF Form'
    portal_type = 'Form'

    __implements__ = ( PortalContent.__implements__,
                       DefaultDublinCoreImpl.__implements__ )

    security = ClassSecurityInfo()

    manage_options = (
        PortalFolder.manage_options[0],
        PortalContent.manage_options[0],) + PortalFolder.manage_options[1:]

    name = ''
    method = 'POST'
    enctype = ''
    _storage = ''
    redirect = ''
    icon = PortalContent.getIcon
    _fields = []

    def __init__(self, id, title='', xmlform=''):
        """
        xmlform default fields in xml format
        """
        self._xmlform = xmlform
        
        PortalFolder.__init__(self, id, title)
        DefaultDublinCoreImpl.__init__(self, title)
        ZMIForm.__init__(self, id, title)
	
    def __call__(self):
        ''' Invokes the default view. '''
        view = _getViewFor(self, 'view')#, 'folderlisting')
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
	    return view()
	    
    security.declareProtected( View, 'view' )
    view = __call__
    index_html = None

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, title, description, fname, faction, fmethod, fenctype, storage, redirect):
        """ edit form """
	self.title = title
	self.description = description
        self.name = fname
        self.action = faction
        self.enctype = fenctype
        self._storage = storage
        self.redirect = redirect
        if fmethod in ['POST', 'GET']:
            self.method = fmethod
        self.reindexObject()

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """ """
	try:
            if hasattr(aq_base(self), '_xmlform') and self._xmlform:
                XMLToForm(self._xmlform, self)
        	del self._xmlform
        except:
            traceback.print_exc()

    security.declareProtected('Change Formulator Forms', 'manage_addField')
    def manage_addField(self, id, title, fieldname, REQUEST=None):
        """ """
        title = title.strip()
        if not title:
            title = id # title is always required, use id if not provided
            self.invokeFactory(fieldname, id, title=title)
        return ''

    security.declareProtected(View, 'FormName')
    def FormName(self):
        """ """
        return self.name

    security.declareProtected(View, 'FormAction')
    def FormAction(self):
        """ """
        return self.action

    security.declareProtected(View, 'FormMethod')
    def FormMethod(self):
        """ """
        return self.method

    security.declareProtected(View, 'FormEnctype')
    def FormEnctype(self):
        """ """
        return self.enctype

    security.declareProtected(View, 'FormRedirect')
    def FormRedirect(self):
        """ """
        return self.redirect

    security.declareProtected(View, 'validate_and_store')
    def validate_and_store(self, REQUEST):
        """ """
        res = self.validate_all(REQUEST)
        storage = self.unrestrictedTraverse(self._storage)
        if (storage.meta_type != 'DataStorage'):
            return

        dict = []
        for grp in self.groups:
		
            f_idx = []
            for field in self.get_fields_in_group(grp):
                f_idx.append([field.getId(), field.Title()])
            dict.append([grp, f_idx])

        res1 = {}
        for key, val in res.items():
            if type(val) == type([]):
                s = ''
                for v in val: s += "%s," % str(v)
                val = '[%s]' % s[:-1]
            res1[key] = val

        storage.append(self, res1, dictionary=dict)
        return res

    security.declareProtected(ModifyPortalContent, 'formOrder')
    def formOrder(self, context, REQUEST, manage_tabs_message):
        """ return Groups form """
        target_action = self.getTypeInfo().getActionById('groups')

        return REQUEST.RESPONSE.redirect( '%s/%s?portal_status_message=%s' % (
            self.absolute_url(), target_action, manage_tabs_message) )


def manage_addFormulatorForm(self, id, title="", xmlform='', REQUEST=None):
    """Add form to folder. """
    id = self._setObject(id, CMFForm(id, title, xmlform))
    
    if REQUEST is not None:
        return self.folder_contents( # XXX: ick!
            self, REQUEST, portal_status_message="CMF form added")

InitializeClass(CMFForm)
